@@ -2,21 +2,34 @@ package Lingua::TokenParse;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.03';
+$VERSION = '0.04';
 
 # NOTE: The {{{ and }}} things are "editor code fold markers".  They
 # are merely a convenience for people who don't care to scroll through
 # reams of source, like me.
 
+# Globals used by the build_combinations method.
+my (@parsed, @new, $prev);
+
 sub new {  # {{{
     my ($class, %args) = @_;
 
     my $self = {
-        word => $args{word} || undef,     # The word to parse!
-        lexicon => $args{lexicon} || {},  # The list of known tokens.
-        parts => [],         # The list of all word parts.
-        combinations => [],  # The list of all possible parts combinations.
-        knowns => {},        # The scored list of the known parts combinations.
+        # The word to parse!
+        word         => $args{word} || undef,
+        # We need to use the length of our word in a few methods.
+        word_length  => exists $args{word} ? length ($args{word}) : undef,
+        # The list of known tokens.
+        lexicon      => $args{lexicon} || {},
+        # The list of all word parts.
+        parts        => [],
+        # The list of all possible parts combinations.
+        combinations => [],
+        # The scored list of the known parts combinations.
+        knowns       => {},
+        # The list of definitions of the known and unknown fragments
+        # in knowns.
+        definitions  => {},
     };
 
     bless $self, $class;
@@ -25,6 +38,7 @@ sub new {  # {{{
         $self->build_parts;
         $self->build_combinations;
         $self->build_knowns;
+        $self->build_definitions;
     }
 
     return $self;
@@ -33,7 +47,10 @@ sub new {  # {{{
 # Accessors {{{
 sub word {  # {{{
     my $self = shift;
-    $self->{word} = shift if @_;
+    if (@_) {
+        $self->{word} = shift;
+        $self->{word_length} = length $self->{word};
+    }
     return $self->{word};
 }  # }}}
 
@@ -60,8 +77,24 @@ sub knowns {  # {{{
     $self->{knowns} = shift if @_;
     return $self->{knowns};
 }  # }}}
+
+sub definitions {  # {{{
+    my $self = shift;
+    $self->{definitions} = shift if @_;
+    return $self->{definitions};
+}  # }}}
 # }}}
 
+sub reset_parse {  # {{{
+    my $self = shift;
+    $self->parts([]);
+    $self->combinations([]);
+    $self->knowns({});
+    $self->definitions({});
+    @parsed = ();
+    @new    = ();
+}  # }}}
+
 sub build_parts {  # {{{
     my $self = shift;
 
@@ -74,8 +107,6 @@ sub build_parts {  # {{{
     }
 }  # }}}
 
-# Globals used by the build_combinations method.
-my (@parsed, @new, $prev);
 sub build_combinations {  # {{{
     my ($self, $i) = @_;
     $i = 0 unless defined $i;
@@ -104,18 +135,15 @@ sub build_combinations {  # {{{
 sub build_knowns {  # {{{
     my $self = shift;
 
-    # Make a familiar combination from each "raw" combination.
+    # Show familiar combinations for each "raw" combination.
     for my $combo (@{ $self->combinations }) {
         my $sum = 0;
 
         # Get the bits of the combination.
         my @chunks = split /\./, $combo;
 
-        # Sum the combination familiarity value and flag the unknowns.
         for (@chunks) {
             # Handle hyphens in lexicon entries.
-            # Thanks for saying, "That's probably how I would do
-            # that.", for this code, Kirsten.  readability++
             my $flag = 0;
             if (exists $self->lexicon->{$_}) {
                 $flag++;
@@ -129,32 +157,43 @@ sub build_knowns {  # {{{
                 $_ = "-$_";
             }
 
-            if ($flag) {
-                $sum++;
-            }
-            else {
-                $_ .= '~';
-            }
+            # Sum the combination familiarity value.
+            $sum++ if $flag;
         }
 
-        # Mash the adjacent unknown bits together.
-        my (@seen, $unknown);
-        for (@chunks) {
-            if (/~$/) {
-                s/~$//;
-                $unknown .= $_;
-            }
-            else {
-                push @seen, $unknown if $unknown;
-                push @seen, $_;
-                $unknown = '';
-            }
-        }
-        push @seen, $unknown if $unknown;
+        $combo = join '.', @chunks;
 
         # Save this combination with the familiarity ratio as the
         # value.
-        $self->knowns->{ join '.', @seen } = $sum / @seen if $sum;
+        $self->knowns->{$combo} = $sum / @chunks
+            if $sum and !exists $self->knowns->{$combo};
+    }
+}  # }}}
+
+sub build_definitions {  # {{{
+    my $self = shift;
+
+    # Make a familiar combination from each "raw" combination.
+    for my $combo (keys %{ $self->knowns }) {
+        # Get the bits of the combination.
+        my @chunks = split /\./, $combo;
+
+        # Save entries with their definitions as the values.
+        for (@chunks) {
+            # Handle hyphens in lexicon entries.
+            if (exists $self->lexicon->{$_}) {
+                $self->definitions->{$_} = $self->lexicon->{$_};
+            }
+            elsif (exists $self->lexicon->{"$_-"}) {
+                $self->definitions->{$_} = $self->lexicon->{"$_-"};
+            }
+            elsif (exists $self->lexicon->{"-$_"}) {
+                $self->definitions->{$_} = $self->lexicon->{"-$_"};
+            }
+            else {
+                $self->definitions->{$_} = undef;
+            }
+        }
     }
 }  # }}}
 
@@ -174,22 +213,49 @@ __END__
 
 =head1 NAME
 
-Lingua::TokenParse - Parse a word into scored, familiar combinations
+Lingua::TokenParse - Parse a word into scored, fragment combinations
 
 =head1 SYNOPSIS
 
   use Lingua::TokenParse;
 
+  my $word = 'partition';
   my %lexicon;
   @lexicon{qw(part i tion on)} = ();
 
   my $obj = Lingua::TokenParse->new(
-      word    => 'partition',
+      word => $word,
       lexicon => \%lexicon,
   );
 
   $obj->output_knowns;
 
+  # Okay.  Now, let's parse a new word.
+  $obj->reset_parse;
+
+  $obj->word('metaphysical');
+
+  $obj->lexicon({
+      'meta-' => 'more comprehensive',
+      'phys'  => 'natural science, singular',
+      '-ic'   => 'being, containing',
+      '-al'   => 'relating to, characterized by',
+  });
+
+  $obj->build_parts;
+  $obj->build_combinations;
+  $obj->build_knowns;
+  $obj->build_definitions;
+
+  $obj->output_knowns;
+
+  print "@$_\n" for @{ $obj->parts };  # This has nice looking output.
+  use Data::Dumper;
+  print Dumper $obj->parts;
+  print Dumper $obj->combinations;
+  print Dumper $obj->knowns;
+  print Dumper $obj->definitions;
+
 =head1 ABSTRACT
 
 This class represents a Lingua::TokenParse object and contains 
@@ -199,7 +265,7 @@ lexicon of known word parts.
 =head1 DESCRIPTION
 
 A word like "partition" is actually composed of a few different word
-parts.  Given a lexicon of known word parts, it is possible to 
+parts.  Given a lexicon of known fragments, it is possible to 
 partition this word into combinations of these (possibly overlapping)
 parts.  Each of these combinations can be given a score, which 
 represents a measure of familiarity.
@@ -207,8 +273,8 @@ represents a measure of familiarity.
 Currently, this familiarity mesasure is a simple ratio of known to 
 unknown parts.
 
-* Please check out the sample code in the eg/ directory for an 
-exciting example of how this module can be used.
+* Please check out the sample code in the distribution's eg/ 
+directory for exciting examples of how this module can be used.
 
 =head1 METHODS
 
@@ -224,6 +290,13 @@ Return a new Lingua::TokenParse object.
 This method will automatically call the partition methods (detailed 
 below) if a word and lexicon are provided.
 
+=head2 reset_parse()
+
+Reset the lists used to parse a word into fragment combinations.
+
+That method must be called prior to reparsing a new word in the same 
+session.
+
 =head2 build_parts()
 
   $obj->build_parts();
@@ -246,10 +319,17 @@ Compute the familiar word part combinations, accessed via the knowns()
 method.
 
 This method handles word parts containing prefix and suffix hyphens,
-which are found in the web1913 dict server.  These hyphens actually 
-"encode" information about what is a syntactically legal word 
-combination.  Which can be used to score (or just throw out bogus 
-combinations).
+which are found in the dict.org sever (with web1913 database).  These
+hyphens actually "encode" information about what is a syntactically 
+legal word combination.  Which can be used to score (or just throw 
+out bogus combinations).
+
+=head2 build_definitions()
+
+  $obj->build_definitions();
+
+Construct a hash of the definitions of the word parts in each 
+combination in the keys of the knowns hash.
 
 =head2 output_knowns()
 
@@ -308,6 +388,14 @@ combinations are kept.
 Note that this method is only useful for fetching, since the knowns
 are computed by the build_knowns() method.
 
+=head2 definitions()
+
+  $definitions = definitions();
+
+The hash reference of the definitions provided for each fragment of 
+the combinations in the knowns hash.  Note that the unknown 
+fragments are defined as an empty string.
+
 =head1 DEPENDENCIES
 
 None
@@ -316,13 +404,23 @@ None
 
 This module uses some clunky, inefficient algorithms.  For instance,
 a 50 letter word (like a medical term) just might take until the end
-of time to parse.  Please write to me with improvements!
+of time to parse and possibly longer.  Please write to me with 
+improvements!
 
 =head1 TO DO
 
-Handle the successor method and related globals correctly.
+Trim the known combinations (and definition list) down to ones that
+concatinate the adjacent unknown fragments together.  Then throw out 
+the ones with known fragments within that "unknown chunk".
 
-Make a knowns hash that maps to definitions.
+Calculate familiarity with more granularity.  Possibly with a
+multidimensional measure.
+
+Output some type of concatinated, known combination definition.
+
+Handle the build_combinations method and related globals correctly.
+
+Compute the time required for a given parse.
 
 Synthesize a term list based on word part (thesaurus) definitions.
 (That is, go in reverse! Non-trivial!)
